''*****************************
''*  16Colour Bitmap          *
''*  (C) 2009 Jim Bagley      *
''*  + some animation demos   *
''*     Peter Jakacki         *
''*****************************

CON

{Hybrid Setup
_clkmode = xtal1 + pll16x
_xinfreq = 6_000_000
modepins = %011_0000
}

'{Proto/Demo Board Setup
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  modepins = %001_0101
'}

{Hydra Setup
_clkmode = xtal1 + pll8x
_xinfreq = 10_000_000
modepins = %011_0000
'}

PAL=%0001
NTSC=%0000
INTERLACED=%0010

x_tiles = 8 '*16=256            ' 128 bytes/line
y_tiles = 12 '*16=192
mode = 0
my_hx = 22+((mode&1)<<1)

x_pixels = x_tiles*16
y_pixels = y_tiles*16

bcnt = 128*191
clockfreq = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ
_1mS  = clockfreq /     1_000 'Divisor for  1mS

  maxSprites    = 8
  maxYSZ        = 24
  maxXSZ        = 16
  spriteSZ      = maxYSZ*maxXSZ

  #0,Heli,Missile,Baddie,Baddie2,Baddie3

OBJ
  tv    : "JB_TV_AMS_01"
  font  : "Font_ATARI"

VAR
  long  frame

  byte  Sprites[16*maxYSZ*maxSprites]
  byte  Shadows[16*maxYSZ*maxSprites]

  byte  SpriteXSZ[maxSprites]
  byte  SpriteYSZ[maxSprites]
  byte  shadowX[maxSprites]
  byte  shadowY[maxSprites]
  byte  ix[8],iy[8],ii[8],spritemode[maxSprites]


PUB Main | i,c,x,y,lc

  tv.start(@tvparams)
  frame~
  print(string(" JB "),0,0,35,-1)
  print(string("PBJ "),0,10,34,-1)
  print(string("OBC "),97,0,35,-1)

  ' Grab areas of the screen to make some sprites
  MakeSprite(Heli,58,106,12,22)
  MakeSprite(Missile,104,113,6,5)
  MakeSprite(Baddie,0,109,8,15)
  CopySprite(Baddie+1,Baddie)
  CopySprite(Baddie+2,Baddie)

  ' now zap this area of the screen for a blank playing field
  bytefill(screen(0,96),0,(158-96)<<6)

  ' Initial sprite positions
  ix[Heli] := 56
  iy[Heli] := 106

  ix[Missile] := 104
  iy[Missile] := 114
  iy[Baddie] := 86

  ix[Baddie+1] := 80
  iy[Baddie+1] := 106

  ix[Baddie+2] := 94
  iy[Baddie+2] := 156

  ' initial speeds
  ii[Heli] := 2
  ii[Baddie] := 2
  ii[Baddie+1] := 2

  repeat
    VSync
    RestoreScreen
    repeat y from 159 to 191    ' scroll this horizontal area
      i := @data+(y<<6)         ' point to line in memory
      c := byte[ i ]            ' save the first byte as we are scrolling left
      bytemove(i,i+1,63)        ' move the line left 2 pixels
      byte[i+63] := c           ' and wrap-around with the original left pixels now on the right
    DrawSprites

    if Collided(Heli,Missile)
       spritemode[Heli] |= 1
    else
       spritemode[Heli] &= !1

    if Collided(Missile,Baddie+1)
      iy[Baddie+1]++


    Vsync
    ' movement update for all the sprites
    ix[missile] += 5
    ix[missile] &= 127

    ix[Heli] += 2
    iy[Heli] += ii[Heli]
    ' keep Mr Heli within this vertical frame
    if iy[Heli] > 159-maxYSZ OR iy[Heli] =< 64
      -ii[Heli]

    if lc&1 == 0                ' slower updates
      repeat i from 1 to 2
        ix[Baddie+i] +=2+i           ' Moving forward
        ix[Baddie+i] &= 127
        iy[Baddie+i] += ii[Baddie+i] ' vertical movement
        -ii[Baddie+i]              ' bounce the heli up and down
    ix[Baddie] += 1
    iy[Baddie] += ii[Baddie]
    ' keep within this vertical frame
    if iy[Baddie] > 159-maxYSZ OR iy[Baddie] =< 64
      -ii[Baddie]

    lc++                        ' loop counter

pub MakeSprite(n,x,y,xsz,ysz) | i
  spriteXSZ[n] := xsz           ' save size properties
  spriteYSZ[n] := ysz
  repeat i from 0 to ysz        ' Q&D screen area grab to sprite table
     bytemove(@sprites[(n*spriteSZ)+(i*maxYSZ)], @data+(x>>1)+((y+i)<<6), xsz>>1)

pub CopySprite(dst,src)
  bytemove(@sprites[dst*spriteSZ],@sprites[src*spriteSZ],spriteSZ)
  spriteXSZ[dst] := spriteXSZ[src]
  spriteYSZ[dst] := spriteYSZ[src]

pub DrawSprite(n,x,y) | i
  if spriteXSZ[n] AND spriteYSZ[n]
    x &= 127                      ' wrap any X given as it may be incrementing past the right side
    repeat i from 0 to spriteYSZ[n] ' Q&D screen area write from sprite table
      if spritemode[n] & 1
        bytefill(screen(x,y+i), $11+i+(i<<4), spriteXSZ[n]>>1)
      else
        bytemove(screen(x,y+i), @sprites[(n*spriteSZ)+(i*maxYSZ)], spriteXSZ[n]>>1)


pub SaveShadow(n,x,y) | i,p
  shadowX[n] := x               ' remember the position of the shadow for later restoration
  shadowY[n] := y
  p := n*spriteSZ               ' speed things up a bit and remember sprite table address
  repeat i from 0 to spriteYSZ[n]
     bytemove(@shadows[p+(i*maxXSZ)], screen(x,y+i), spriteXSZ[n]>>1)

pub RestoreShadow(n) | i,p
  if shadowx[n] + shadowy[n]    ' another Q&D to prevent restore the first time the sprite is drawn
    p := n*spriteSZ
    repeat i from 0 to spriteYSZ[n]
       bytemove(screen(shadowX[n], shadowY[n]+i), @shadows[p+(i*maxXSZ)], spriteXSZ[n]>>1)


' full screen synchronized drawing

pub DrawSprites | i
'  RestoreScreen
  SaveShadows
  repeat i from maxSprites-1 to 0        ' update the sprites in priority - 0 = top
    DrawSprite(i,ix[i],iy[i])

pub SaveShadows | i
  repeat i from 0 to maxSprites-1        ' update the sprites
    SaveShadow(i,ix[i],iy[i])


pub RestoreScreen | i           ' part of the "draw em all, restore them all" plan for later
  repeat i from 0 to maxSprites-1
    RestoreShadow(i)


pub Collided(sprite1,sprite2)
  if ix[sprite1] => ix[sprite2] and ix[sprite1] =< ix[sprite2]+spriteXSZ[sprite2]
    if iy[sprite1] => iy[sprite2] and iy[sprite1] =< iy[sprite2]+spriteYSZ[sprite2]
      result := true


pub screen(x,y)                 ' point to the screen memory which consists of 2 pixels/byte, 64 bytes/line
  result := @data+(x>>1)+(y<<6)


PUB plot(x,y,c)
  if(x<0 or x=>x_pixels or y<0 or y=>y_pixels)
    return
  c&=15
  if(x&1)
    data[y<<6+x>>1]:=(data[y<<6+x>>1]&$f0)+c
  else
    data[y<<6+x>>1]:=(data[y<<6+x>>1]&$0f)+(c<<4)



PUB print(s,x,y,c,b)            ' s=string, x,y obvious c=colour,b=background colour -1=transparent background
  repeat while byte[s][0]
    prtchr(byte[s][0],x,y,c,b)
    s++
    x+=8

PUB prtchr(ch,x,y,c,b) | f,xx,yy
  f:=font.GetPtrToFontTable+(ch<<3)
  repeat yy from 0 to 7
    repeat xx from 0 to 7
      if(byte[f][0]&(|<(7-xx)))
        plot(x+xx,y+yy,c)
      else
        if(b<>-1)
          plot(x+xx,y+yy,b)
    f++



PUB VSync
  repeat until VSyncFlag<>1
  repeat until VSyncFlag==1

PUB ms(Period)
  if Period                                    '1 mS
    waitcnt(_1mS * Period + cnt)

DAT

tvparams      long    0               'status
              long    1               'enable
              long    modepins        'pins ' PROTO/DEMO BOARD = %001_0101 ' HYDRA = %011_0000
              long    mode            'mode
              long    x_tiles         'hc
              long    y_tiles         'vc
              long    my_hx           'hx
VSyncFlag     long    0
nextline      long    0                        
disp_ptr      long    @data+$10
bordercolour  long    $02
              long    @palette+$10

palette       byte    $02,$07,$28,$8d,$ba,$1a,$db,$3d,$8f,$5f,$3f,$5b,$d8,$04,$02,$02

data          file    "mrheli.bit"



